#include "input.inc"
!--------------------------------------------!
! author Simone Meloni, s.meloni@caspur.it   !
! InterUniversity consortium                 !
! application of Supercomputing for          !
! University and Research                    !
! CASPUR                                     !
!                                            !
! - ver 0.1a 14/10/2008                      !
!                                            !
!--------------------------------------------!

       program gr
! radial distribution function

       implicit none
! constants
       integer numAtoms
       parameter (numAtoms = NUM_ATOMS)

! dinamical variables & observables 
       real*8 positions(3,numAtoms)
       real*8 simBox
       real*8 g(GR_BINS)

! service variables
       integer iter, skipFrames, useFrames
       integer numNearestNeighbor(numAtoms), &
               nearestNeighborList(MAX_NEAREST_NEIGHBOR, numAtoms)
       real*8  nearestNeighborDist(4, MAX_NEAREST_NEIGHBOR, numAtoms)

       logical lastIter
       integer thisBin, i, j
       real*8  distIJ, density, nPairs, volume
       real*8 grRange2, binSize, pi
       parameter (grRange2=GR_RANGE*GR_RANGE, binSize = dble(GR_RANGE)/dble(GR_BINS), pi=2.d0*asin(1.d0))

! force field parameters 
       include 'forceField.f90'


! timing 
! temporary disabled for problem on measuring wallclock 
! in F90
       call MyTiming(0)

! read simulation box
       open(10, file="box.xyz", status="OLD")
       read(10,*) simBox
       close(10)


! g(r): averaging over configurations
       iter = 0
       lastIter = .false.
       g(:) = 0.d0
       do 
         iter = iter + 1
         call readFrame(numAtoms, simBox, positions, lastIter)
         if (lastIter) exit
         
         if (iter .gt. SKIP_FRAMES .and. iter .le. SKIP_FRAMES + MAX_GR_ITER) then

#if LISTS == NSQUARE
            call nSquareLists(grRange2, numAtoms, simBox, positions, &
               numNearestNeighbor, nearestNeighborList, nearestNeighborDist)
#elif LISTS == LINKED_CELL
            call linkedCellLists(grRange2, numAtoms, simBox, positions, &
               numNearestNeighbor, nearestNeighborList, nearestNeighborDist)
#else
	    write(0,*)"you selected a wrong type of list"
            stop
#endif /* LISTS */

            do i = 1, numAtoms
               do j = 1, numNearestNeighbor(i)
                  distIJ = sqrt(nearestNeighborDist(4,j,i))
                  thisBin = distIJ / binSize + 1
                  g(thisBin) = g(thisBin) + 2
               end do
            end do
         else if (iter .gt. SKIP_FRAMES + MAX_GR_ITER) then 
	    exit
         end if

       end do

! normalization

! by shell
       density = numAtoms/(simBox**3)


       do i = 1, GR_BINS
          nPairs = 4.d0/3.d0*pi*((i+1)**3 - i**3)*(binSize**3)*density
          g(i) = g(i) / nPairs
       end do

! over atoms and frames
       g = g /(numAtoms*MAX_GR_ITER)
          

! final timing
! temporary disable for problem on measuring wallclock 
! in F90

       open(10, file="gr", status="unknown")
       do i = 1, GR_BINS
          write(10,'(f10.4, 1x, e15.5)') i*binSize, g(i)
       end do

       call MyTiming(iter)

      end
