#include "input.inc"
!--------------------------------------------!
! author Simone Meloni, s.meloni@caspur.it   !
! InterUniversity consortium                 !
! application of Supercomputing for          !
! University and Research                    !
! CASPUR                                     !
!                                            !
! - ver 0.1a 14/10/2008                      !
!                                            !
!--------------------------------------------!

       program sk
! radial distribution function

       implicit none
! constants
       integer numAtoms
       parameter (numAtoms = NUM_ATOMS)

! dinamical variables & observables 
       real*8 positions(3,numAtoms)
       real*8 simBox
       real*8 kR, sR(SK_BINS)

! service variables
       integer iter, skipFrames, useFrames

       logical lastIter
       integer i, j, l, pairs
       real*8  dist(3), k(3)
       real*8  pi
       parameter (pi=2.d0*asin(1.d0))

! force field parameters 
       include 'forceField.f90'


! timing 
! temporary disabled for problem on measuring wallclock 
! in F90
       call MyTiming(0)

! read simulation box
       open(10, file="box.xyz", status="OLD")
       read(10,*) simBox
       close(10)


! g(r): averaging over configurations
       iter = 0
       lastIter = .false.
       k(1) = 2*pi*KX
       k(2) = 2*pi*KY
       k(3) = 2*pi*KZ
       sR = 0.d0
       pairs = numAtoms * (numAtoms-1) / 2
       open(10, file="sk", status="unknown") 
       do 
         iter = iter + 1
         call readFrame(numAtoms, simBox, positions, lastIter)
         if (lastIter) exit
         
         if (iter .gt. SKIP_FRAMES .and. iter .le. SKIP_FRAMES + MAX_SK_ITER) then

            do i = 1, numAtoms
               do j = i+1, numAtoms
                 dist(:) = positions(:,i) - positions(:,j)
                 dist(:) = dist(:) - simBox * nint(dist(:)/simBox)

                 kR = dot_product(k, dist) 
                 do l = 1, SK_BINS
                    sR(l) = sR(l) + cos(dble(l)*kR/dble(SK_BINS)) 
                 end do
               end do
            end do


         else if (iter .gt. SKIP_FRAMES + MAX_SK_ITER) then 
	    exit
         end if

       end do

       write(10,'(e15.5)') sR/(pairs*MAX_SK_ITER)

       call MyTiming(iter)

      end
