#include "input.inc"
!--------------------------------------------!
! author Simone Meloni, s.meloni@caspur.it   !
! InterUniversity consortium                 !
! application of Supercomputing for          !
! University and Research                    !
! CASPUR                                     !
!                                            !
! - ver 0.1 5/3/2006                         !
!                                            !
!--------------------------------------------!

#if LISTS == NSQUARE || LISTS == VERLET
! compute interaction list with a O(N^2) algorithm
      subroutine nSquareLists(cutoffSquare, numAtoms, simBox, positions, &
         numNearestNeighbor, nearestNeighborList, nearestNeighborDist)
      implicit none
! arguments
      real*8 cutoffSquare
      integer numAtoms
      real*8 positions(3, numAtoms)
      real*8 simBox
      integer numNearestNeighbor(numAtoms), &
              nearestNeighborList(MAX_NEAREST_NEIGHBOR, numAtoms)
      real*8  nearestNeighborDist(4, MAX_NEAREST_NEIGHBOR, numAtoms)
! variables
      integer i , j, k, m
      real*8 dist(3), distSquare

      do i = 1, numAtoms
         numNearestNeighbor(i) = 0
         do j = i+1, numAtoms
            dist(:) = positions(:,i) - positions(:,j)
            dist(:) = dist(:) - simBox * nint(dist(:)/simBox)
            distSquare = dot_product(dist, dist)
            if (distSquare <= cutoffSquare) then
               numNearestNeighbor(i) = numNearestNeighbor(i) + 1 
               nearestNeighborList(numNearestNeighbor(i), i) = j
               nearestNeighborDist(1:3, numNearestNeighbor(i), i) = &
                  dist(:)
               nearestNeighborDist(4, numNearestNeighbor(i), i) = &
                  distSquare
            end if
         end do
      end do
#ifdef DEBUG_LISTS
      do i = 1, numAtoms 
         write(0,*) "atoms = ", i, ", num nearest neigbor = ", numNearestNeighbor(i)
         write(0,fmt="(6(1x, i6))") nearestNeighborList(1:numNearestNeighbor(i), i)
      end do

      stop
#endif
      return 
      end
#endif


#if LISTS == LINKED_CELL || LISTS == LINKED_CELL_VERLET

#   ifdef REORDERING
      module Reordering_mod
      integer, allocatable:: pivotting(:)
      real*8, allocatable:: scratch(:,:)
      end module Reordering_mod
#   endif

      subroutine  linkedCellLists(cutoffSquare, numAtoms, simBox, positions, &
          numNearestNeighbor, nearestNeighborList, nearestNeighborDist)
#   ifdef REORDERING
      use reordering_mod
#   endif
      implicit none
! arguments
      real*8 cutoffSquare
      integer numAtoms
      real*8 positions(3, numAtoms)
      real*8 simBox
      integer numNearestNeighbor(numAtoms), &
              nearestNeighborList(MAX_NEAREST_NEIGHBOR, numAtoms)
      real*8  nearestNeighborDist(4, MAX_NEAREST_NEIGHBOR, numAtoms)
! variables
      integer i , j, k, l, m, n, o
      real*8 dist(3), distSquare
      logical, save :: wasAlreadyCalled = .FALSE.
      integer, save :: numLinkedCell = 0
      real*8, save :: linkedCellSize = 0.d0
      integer, allocatable, save:: atomsInLinkedCell(:,:,:,:),&
         numAtomsInLinkedCell(:,:,:),&
         linkedCell(:,:,:,:,:)
      integer thisLinkedCell, progressiveIdx
      integer belongingToCell(3)
      integer atom1Identity, atom2Identity
      integer myLinkedCellA, myLinkedCellB, myLinkedCellC
      real*8 myPos(3)

      if (wasAlreadyCalled .eqv. .FALSE.) then
#   ifdef REORDERING
         allocate(pivotting(numAtoms), scratch(3, numAtoms))
#   endif
! building linked cells (valid only for a constant volume job)
         numLinkedCell = int(simBox/dsqrt(cutoffSquare))
         linkedCellSize = simBox / numLinkedCell
         allocate(atomsInLinkedCell(MAX_NEAREST_NEIGHBOR, numLinkedCell, numLinkedCell, numLinkedCell))
         allocate(numAtomsInLinkedCell(numLinkedCell, numLinkedCell, numLinkedCell))
         allocate(linkedCell(3,13, numLinkedCell, numLinkedCell, numLinkedCell))
         do i = 1, numLinkedCell
            do j = 1, numLinkedCell
               do k = 1, numLinkedCell
                  progressiveIdx = 0
                  do m = -1, 1
                     do n = -1, 1
                        progressiveIdx = progressiveIdx + 1
                        thisLinkedCell = i - 1 
                        if (thisLinkedCell < 1) then 
                           thisLinkedCell = numLinkedCell
                        else if (thisLinkedCell > numLinkedCell) then 
                           thisLinkedCell = 1
                        endif
                        linkedCell(1,progressiveIdx, i, j, k) = thisLinkedCell 
                        thisLinkedCell = j + m
                        if (thisLinkedCell < 1) then 
                           thisLinkedCell = numLinkedCell
                        else if (thisLinkedCell > numLinkedCell) then 
                           thisLinkedCell = 1
                        endif
                        linkedCell(2,progressiveIdx, i, j, k) = thisLinkedCell 
                        thisLinkedCell = k + n
                        if (thisLinkedCell < 1) then 
                           thisLinkedCell = numLinkedCell
                        else if (thisLinkedCell > numLinkedCell) then 
                           thisLinkedCell = 1
                        endif
                        linkedCell(3,progressiveIdx, i, j, k) = thisLinkedCell
                     enddo
                  enddo
                  do m = 0, 1
                     do n = -1, m
                        if ((m /= 0) .or. (n /= 0) ) then 
                           progressiveIdx = progressiveIdx + 1
                           linkedCell(1,progressiveIdx, i, j, k) = i 

                           thisLinkedCell = j + m
                           if (thisLinkedCell < 1) then 
                              thisLinkedCell = numLinkedCell
                           else if (thisLinkedCell > numLinkedCell) then 
                              thisLinkedCell = 1
                           endif
                           linkedCell(2,progressiveIdx, i, j, k) = thisLinkedCell 
                           thisLinkedCell = k + n
                           if (thisLinkedCell < 1) then 
                              thisLinkedCell = numLinkedCell
                           else if (thisLinkedCell > numLinkedCell) then 
                              thisLinkedCell = 1
                           endif
                           linkedCell(3,progressiveIdx, i, j, k) = thisLinkedCell
                        end if
                     enddo
                  enddo
                  
               enddo
            enddo
         enddo
         wasAlreadyCalled = .TRUE.
      endif


! assigning atoms to linked cells
      numAtomsInLinkedCell = 0
      AtomsInLinkedCell = 0
      do i = 1, numAtoms 
         myPos = positions(:,i)  / simBox
         ! positions contained between [0, simBox)
         myPos = myPos - nint(myPos) + 0.5 
         belongingToCell =  int(myPos * simBox / linkedCellSize) + 1
         numAtomsInLinkedCell(belongingToCell(1), belongingToCell(2), belongingToCell(3)) =&
            numAtomsInLinkedCell(belongingToCell(1), belongingToCell(2), belongingToCell(3)) + 1
         progressiveIdx =  numAtomsInLinkedCell(belongingToCell(1), belongingToCell(2), belongingToCell(3))
          AtomsInLinkedCell(progressiveIdx, belongingToCell(1), belongingToCell(2), belongingToCell(3)) = i
      end do

#   ifdef REORDERING
      progressiveIdx = 0
      do i = 1, numLinkedCell
         do j = 1, numLinkedCell
            do k = 1, numLinkedCell
               do l = 1, numAtomsInLinkedCell(i, j, k)
                  progressiveIdx = progressiveIdx + 1
                  pivotting(progressiveIdx) = AtomsInLinkedCell(l, i, j, k)
                  AtomsInLinkedCell(l, i, j, k) = progressiveIdx
               end do
            end do
         end do
      end do

      call reordering(numAtoms, positions) 
#   endif

! computing interaction lists among atoms belonging to the linked cells
      numNearestNeighbor = 0
      do i = 1, numLinkedCell
         do j = 1, numLinkedCell
            do k = 1, numLinkedCell
               do l = 1, numAtomsInLinkedCell(i, j, k)
                  atom1Identity = AtomsInLinkedCell(l, i, j, k)
! computing intra-cell interaction list
                  do m = l + 1, numAtomsInLinkedCell(i, j, k)
                     atom2Identity = AtomsInLinkedCell(m, i, j, k)
                     dist(:) = positions(:,atom1Identity) - &
                        positions(:,atom2Identity)
                     dist(:) = dist(:) - simBox * nint(dist(:)/simBox)
                     distSquare = dot_product(dist, dist)
                     if (distSquare <= cutoffSquare) then
                        numNearestNeighbor(atom1Identity) = numNearestNeighbor(atom1Identity) + 1
                        nearestNeighborList(numNearestNeighbor(atom1Identity), atom1Identity) = atom2Identity
                        nearestNeighborDist(1:3, numNearestNeighbor(atom1Identity), atom1Identity) = &
                           dist(:)
                        nearestNeighborDist(4, numNearestNeighbor(atom1Identity), atom1Identity) = &
                           distSquare
                     end if
                  end do

! computing inter-cell interaction list
                  do o = 1, 13
                     myLinkedCellA = linkedCell(1, o, i, j, k)
                     myLinkedCellB = linkedCell(2, o, i, j, k)
                     myLinkedCellC = linkedCell(3, o, i, j, k)
                     do m = 1, numAtomsInLinkedCell(myLinkedCellA, myLinkedCellB, myLinkedCellC)
                        atom2Identity = AtomsInLinkedCell(m, myLinkedCellA, myLinkedCellB, myLinkedCellC)
                        dist(:) = positions(:,atom1Identity) - &
                           positions(:,atom2Identity)
                        dist(:) = dist(:) - simBox * nint(dist(:)/simBox)
                        distSquare = dot_product(dist, dist)
                        if (distSquare <= cutoffSquare) then
                           numNearestNeighbor(atom1Identity) = numNearestNeighbor(atom1Identity) + 1
                           nearestNeighborList(numNearestNeighbor(atom1Identity), atom1Identity) = atom2Identity
                           nearestNeighborDist(1:3, numNearestNeighbor(atom1Identity), atom1Identity) = &
                              dist(:)
                           nearestNeighborDist(4, numNearestNeighbor(atom1Identity), atom1Identity) = &
                              distSquare
                        end if
                     end do
                  end do

               end do
            end do
         end do
      end do

#ifdef DEBUG_LISTS
      do i = 1, numAtoms 
         write(0,*) "atoms = ", i, ", num nearest neigbor = ", numNearestNeighbor(i)
         write(0,fmt="(6(1x, i6))") nearestNeighborList(1:numNearestNeighbor(i), i)
      end do

      stop
#endif

      return
      end
#endif


#if LISTS == VERLET ||  LISTS == LINKED_CELL_VERLET
      subroutine  interactionLists(cutoffSquare, numAtoms, simBox, positions, &
          numVerletNeighbor, verletNeighborList, &
          numNearestNeighbor, nearestNeighborList, nearestNeighborDist)
      implicit none
! arguments
      real*8 cutoffSquare
      integer numAtoms
      real*8 positions(3, numAtoms)
      real*8 simBox
      integer numVerletNeighbor(numAtoms), &
              verletNeighborList(MAX_NEAREST_NEIGHBOR, numAtoms)
      integer numNearestNeighbor(numAtoms), &
              nearestNeighborList(MAX_NEAREST_NEIGHBOR, numAtoms)
      real*8  nearestNeighborDist(4, MAX_NEAREST_NEIGHBOR, numAtoms)
! variables
      integer i , j, k, l, m
      real*8 dist(3), distSquare
      logical wasAlreadyCalled /.FALSE./
      save wasAlreadyCalled

      do i = 1, numAtoms
         numNearestNeighbor(i) = 0
         do j = 1, numVerletNeighbor(i)
            l = verletNeighborList(j, i)
            dist(:) = positions(:,i) - positions(:,l)
            dist(:) = dist(:) - simBox * nint(dist(:)/simBox)
            distSquare = dot_product(dist, dist)
            if (distSquare <= cutoffSquare) then
               numNearestNeighbor(i) = numNearestNeighbor(i) + 1 
               nearestNeighborList(numNearestNeighbor(i), i) = l
               nearestNeighborDist(1:3, numNearestNeighbor(i), i) = &
                  dist(:)
               nearestNeighborDist(4, numNearestNeighbor(i), i) = &
                  distSquare
            end if
         end do
      end do   

#ifdef DEBUG_LISTS
      do i = 1, numAtoms 
         write(0,*) "atoms = ", i, ", num nearest neigbor = ", numNearestNeighbor(i)
         write(0,fmt="(6(1x, i6))") nearestNeighborList(1:numNearestNeighbor(i), i)
      end do

      stop
#endif

      return 
      end
#endif

#if (LISTS == LINKED_CELL || LISTS == LINKED_CELL_VERLET) && defined(REORDERING)
      subroutine reordering(numAtoms, vector)
      use Reordering_mod
      implicit none
! arguments
      integer numAtoms
      real*8 vector(3, numAtoms)
! variables
      integer i, j

      do i = 1, numAtoms
         do j = 1, 3
            scratch(j, i) = vector(j, i) 
         end do
      end do

      do i = 1, numAtoms
         do j = 1, 3
            vector(j, i) = scratch(j, pivotting(i))
         end do
      end do

      return 
      end

#endif
