#include "input.inc"
!--------------------------------------------!
! author Simone Meloni, s.meloni@caspur.it   !
! InterUniversity consortium                 !
! application of Supercomputing for          !
! University and Research                    !
! CASPUR                                     !
!                                            !
! - ver 0.1b 9/4/2005                        !
!                                            !
!--------------------------------------------!

       program MoDERN
! MOlecular Dynamics Example Really Naive

       implicit none

! constants
       integer numAtoms, maxIter
       real*8  timeStep, mass
       parameter (numAtoms = NUM_ATOMS, maxIter=MAX_ITER)
       parameter (timeStep = TIME_STEP, mass = MASS)

! dinamical variables & observables 
       real*8 positions(3,numAtoms), velocities(3,numAtoms), &
             forces(3,numAtoms)
       real*8 simBox
       real*8 potEnergy, kinEnergy
! service variables
#if (LISTS == VERLET) || (LISTS == LINKED_CELL_VERLET)
       integer numVerletNeighbor(numAtoms), &
               verletNeighborList(MAX_NEAREST_NEIGHBOR, numAtoms)
       real*8 verletNeighborDist(4, MAX_NEAREST_NEIGHBOR, numAtoms)
#endif
       integer numNearestNeighbor(numAtoms), &
               nearestNeighborList(MAX_NEAREST_NEIGHBOR, numAtoms)
       real*8  nearestNeighborDist(4, MAX_NEAREST_NEIGHBOR, numAtoms)

! loop & index variables
       integer i,j,ki, iter!,bufferingIdx
! functions
       real*8 ComputeKinetic
       external ComputeKinetic
! damp buffer
       real*8 outBuffer(3, BUFFER_SIZE)

! force field parameters 
       include 'forceField.f90'

! Print MoDERN logo
       call PrintLogo()

! timing 
! temporary disabled for problem on measuring wallclock 
! in F90
       call MyTiming(0)

! initialize positions and velocities datastructures
       call MDInit(numAtoms, positions, velocities, simBox, mass)

! before starting molecular dynamics (MD) loop it needs to compute forces
! used to update velocities to the first fisrt (first square ;-) half time step
#if LISTS == NSQUARE
       call nSquareLists(cutoffSquare, numAtoms, simBox, positions, &
          numNearestNeighbor, nearestNeighborList, nearestNeighborDist)
#elif LISTS == VERLET
       call nSquareLists(BUFFER_RATIO * cutoffSquare, numAtoms, simBox, &
          positions, numVerletNeighbor, verletNeighborList, verletNeighborDist)
       call interactionLists(cutoffSquare, numAtoms, simBox, positions, &
          numVerletNeighbor, verletNeighborList, &
          numNearestNeighbor, nearestNeighborList, nearestNeighborDist)
#elif LISTS == LINKED_CELL
       call linkedCellLists(cutoffSquare, numAtoms, simBox, positions, &
          numNearestNeighbor, nearestNeighborList, nearestNeighborDist)
#   ifdef REORDERING
       call reordering(numAtoms, velocities)
#   endif
#elif LISTS == LINKED_CELL_VERLET
       call linkedCellLists(BUFFER_RATIO * cutoffSquare, numAtoms, simBox, positions, &
          numVerletNeighbor, verletNeighborList, verletNeighborDist)
#   ifdef REORDERING
       call reordering(numAtoms, velocities)
#   endif
       call interactionLists(cutoffSquare, numAtoms, simBox, positions, &
          numVerletNeighbor, verletNeighborList, &
          numNearestNeighbor, nearestNeighborList, nearestNeighborDist)
#endif /* LISTS */
       call ComputeForces(numAtoms, potEnergy, forces, numNearestNeighbor, &
         nearestNeighborList, nearestNeighborDist)
        
       kinEnergy =  ComputeKinetic(numAtoms, velocities, mass)

! damp out observables
       call DumpObservables(0, kinEnergy, potEnergy)

! MD loop
       do iter = 1, maxIter
         call PrintProgress(iter)
! first half step velocity integration
         call UpdateVelocities(numAtoms, velocities, &
                                 forces, mass, timeStep/2.d0)
! position integration
         call UpdatePositions(numAtoms, positions, velocities, timeStep)
! force calculation
#if LISTS == NSQUARE
         call nSquareLists(cutoffSquare, numAtoms, simBox, positions, &
            numNearestNeighbor, nearestNeighborList, nearestNeighborDist)
#elif LISTS == VERLET
         if (mod(iter, VERLET_FREQUENCY) == 0) then 
            call nSquareLists(BUFFER_RATIO * cutoffSquare, numAtoms, simBox, &
               positions, numVerletNeighbor, verletNeighborList, &
               verletNeighborDist)
         endif
         call interactionLists(cutoffSquare, numAtoms, simBox, positions, &
            numVerletNeighbor, verletNeighborList, &
            numNearestNeighbor, nearestNeighborList, nearestNeighborDist)
#elif LISTS == LINKED_CELL
         call linkedCellLists(cutoffSquare, numAtoms, simBox, positions, &
           numNearestNeighbor, nearestNeighborList, nearestNeighborDist)
#   ifdef REORDERING
          call reordering(numAtoms, velocities)
#   endif
#elif LISTS == LINKED_CELL_VERLET
         if (mod(iter, VERLET_FREQUENCY) == 0) then
            call linkedCellLists(BUFFER_RATIO * cutoffSquare, numAtoms, simBox, positions, &
               numVerletNeighbor, verletNeighborList, verletNeighborDist)
#   ifdef REORDERING
               call reordering(numAtoms, velocities)
#   endif
         end if
         call interactionLists(cutoffSquare, numAtoms, simBox, positions, &
            numVerletNeighbor, verletNeighborList, &
            numNearestNeighbor, nearestNeighborList, nearestNeighborDist)
#endif /* LISTS */
         call ComputeForces(numAtoms, potEnergy, forces, numNearestNeighbor, &
           nearestNeighborList, nearestNeighborDist)
! second half step velocity integration
         call UpdateVelocities(numAtoms, velocities, &
                                 forces, mass, timeStep/2.d0)
! compute kinetic energy
         kinEnergy =  ComputeKinetic(numAtoms, velocities, mass)

! damp out observables
         call DumpObservables(iter, kinEnergy, potEnergy)
         call DumpTrajectory (numAtoms, simBox, iter, positions, velocities)
       end do

! final timing
! temporary disable for problem on measuring wallclock 
! in F90
       call MyTiming(iter)

      end
