#include "input.inc"
!--------------------------------------------!
! author Simone Meloni, s.meloni@caspur.it   !
! InterUniversity consortium                 !
! application of Supercomputing for          !
! University and Research                    !
! CASPUR                                     !
!                                            !
! - ver 0.3 5/10/2006                        !
!                                            !
!--------------------------------------------!

      subroutine ComputeForces(numAtoms, potEnergy, forces, numNearestNeighbor, &
         nearestNeighborList, nearestNeighborDist)
      implicit none
! arguments
      integer numAtoms
      real*8 forces(3,numAtoms)
      real*8 potEnergy
      integer numNearestNeighbor(numAtoms), &
              nearestNeighborList(MAX_NEAREST_NEIGHBOR, numAtoms)
      real*8  nearestNeighborDist(4, MAX_NEAREST_NEIGHBOR, numAtoms) 
! variables
      integer i , j, k
      integer neighbor
      real*8 forceFixPart
#if METHOD == INTERPOLATED
      logical, save:: wasAlreadyCalled = .FALSE. 
      integer, save:: interpolationPoints
      real*8, allocatable, save:: potInt(:), forceInt(:)
      real*8 r
      integer point
      integer interpolPoint
      real*8 weightPrevious, weightSuccessive
      real*8, parameter:: dRSquare = DR * DR
#endif
      include "forceField.f90"


#if METHOD == INTERPOLATED
      if (wasAlreadyCalled .eqv. .FALSE. ) then 
         interpolationPoints = int(cutoffSquare / dRSquare)
         allocate(potInt(0:interpolationPoints), forceInt(0:interpolationPoints))

         potInt(0) = 10.0d0**range(r)
         forceInt(0) = 10.0d0**range(r)
         do i = 1, interpolationPoints - 1
            r = i  * dRSquare
            potInt(i) = Pot(r)
            forceInt(i) = Force(r)
         end do
         potInt(interpolationPoints) = 0.d0
         forceInt(interpolationPoints) = 0.d0
         
         wasAlreadyCalled = .TRUE.
#   if DEBUG_POTENTIAL
      do i = 1, interpolationPoints
         write(0,*) sqrt( i * dRSquare), potInt(i), forceInt(i) 
      end do
      stop
#   endif
      end if
#endif

      potEnergy = 0.d0
      forces(:,:) = 0.d0
      do i = 1, numAtoms
         do j = 1, numNearestNeighbor(i)
            neighbor = nearestNeighborList(j, i)
#if METHOD == ANALYTICAL
            potEnergy = potEnergy + &
               Pot(nearestNeighborDist(4, j, i))
            forceFixPart = Force(nearestNeighborDist(4, j, i))
#elif METHOD == INTERPOLATED
            point = nearestNeighborDist(4, j, i) / dRSquare
            interpolPoint = int(point)
            weightPrevious =  (interpolPoint + 1) - &
               point  
            weightSuccessive = point - &
              interpolPoint 
            potEnergy = potEnergy + &
               weightPrevious * potInt(interpolPoint) + &
               weightSuccessive * potInt(interpolPoint+1)
            forceFixPart = &
               weightPrevious * forceInt(interpolPoint) + &
               weightSuccessive * forceInt(interpolPoint+1)
#endif
            forces(:,i) = forces(:,i) + &
               forceFixPart * nearestNeighborDist(1:3,j,i)
            forces(:,neighbor) = forces(:,neighbor) - &
               forceFixPart * nearestNeighborDist(1:3,j,i)
         end do
      end do
      return 
      end

