!--------------------------------------------!
! author Simone Meloni, s.meloni@caspur.it   !
! InterUniversity consortium                 !
! application of Supercomputing for          !
! University and Research                    !
! CASPUR                                     !
!                                            !
! - ver 0.1b 9/4/2005                        !
!                                            !
!--------------------------------------------!

! initialize positions and velocities datastructures
       subroutine MDInit(numAtoms, positions, velocities, simBox, mass)
       implicit none
! arguments
       integer numAtoms
       real*8 positions(3,numAtoms), velocities(3, numAtoms), vCOM(3)
       real*8 simBox, mass
! variables
       integer i, j
       real*8 kin_Tmp,kin_target
       real*8 boltzman
       parameter (boltzman=8.6173855e-5)
! functions
       real*8 MyRand, ComputeKinetic
       external MyRand, ComputeKinetic

       open(unit=10, file="positions.xyz", status="old")
       read(10,*)positions
       close(10)
       open(unit=10, file="box.xyz", status="old")
       read(10,*)simBox
       close(10)

       kin_target = TEMPERATURE / 2.d0 * numAtoms * 3.d0 * boltzman 
#ifdef MAXWELL_BOLTZMAN
       call gaussianNoise(numAtoms, 0.d0, kin_target/MASS, velocities)
#elif 
       call whitwhiteeNoise(nParticles, 0.d0, temperature, vel)
#endif
       vCOM = 0.d0
       do i = 1, numAtoms
             vCOM(:) = vCOM(:) + velocities(:,i)
       end do
       vCOM = vCOM/dble(numAtoms)

       do i = 1, numAtoms
          velocities(:,i) = velocities(:,i) - vCOM(:)
       end do
       kin_Tmp =  ComputeKinetic(numAtoms, velocities, mass)
       velocities(:,:) = velocities(:,:) * sqrt(kin_target/kin_Tmp)

       return
       end

! wrapper to a pseudorandom number generator
! I decided to use standard generator from F90
! but you can change it with some better generator 
! or some other available on your unix box (rand, ANSI C function
! would require combining C and Fortan, not that difficult but... do you know
! how to do it?)
       function MyRand()
       implicit none
       real*8 MyRand
       
       call Random_Number(MyRand)
       
       return
       end
   


! compute kinetic energy
      function ComputeKinetic(numAtoms, velocities, mass)
      implicit none
! return value
      real*8 ComputeKinetic
! arguments 
      integer numAtoms
      real*8 velocities(3,numAtoms)
      real*8 mass
! variables
      integer i, j
      real*8 halfMass
  
      halfMass = 0.5 * mass

! this is the same as calling ddot (level 1 BLAS function) and then
! multiply by halfMass
      ComputeKinetic = 0.d0
      do i = 1, numAtoms
         do j = 1, 3
            ComputeKinetic = ComputeKinetic + &
                           halfMass * velocities(j,i) * velocities(j,i)
         end do
      end do

      return
      end



      subroutine DumpObservables(iter, kinEnergy, potEnergy)
! damp out observables
      implicit none
! arguments
      integer iter
      real*8 kinEnergy, potEnergy
! variables
      integer bufferingIdx, j, k
      real*8 outBuffer(3, 0:BUFFER_SIZE-1)
      save outBuffer

      if(iter == 0) then
         open(unit=22, file="observables", status="unknown")
         write(22,*)"# col 1: Kinetic Energy"
         write(22,*)"# col 2: Potential Energy"
         write(22,*)"# col 3: Total Energy"
         outBuffer(1,0) = kinEnergy
         outBuffer(2,0) = potEnergy
         outBuffer(3,0) = kinEnergy + potEnergy
         return
      else
         bufferingIdx = mod(iter,BUFFER_SIZE)
         if (bufferingIdx == 0) then 
            do j = 0, BUFFER_SIZE - 1
               write(22,FMT='(i6, 3(1x,e15.6))')iter-BUFFER_SIZE+j,&
                                         (outBuffer(k,j), k=1,3)
            end do
         end if
         outBuffer(1,bufferingIdx) = kinEnergy
         outBuffer(2,bufferingIdx) = potEnergy
         outBuffer(3,bufferingIdx) = kinEnergy + potEnergy
         if (iter == MAX_ITER) then
#if (MAX_ITER >= MAX_BUFFER)
            do j = 0, bufferingIdx
               write(22,FMT='(i6, 3(1x,e15.6))')iter-bufferingIdx+j,&
                                         (outBuffer(k,j), k=1,3)
            end do
#else
            do j = 0, bufferingIdx
               write(22,FMT='(i6, 3(1x,e15.6))')j,&
                                         (outBuffer(k,j), k=1,3)
            end do
#endif
         end if
            
      endif

      return
      end

      subroutine DumpTrajectory (numAtoms, simBox, iter, positions, velocities)
      implicit none
! arguments
      integer numAtoms, iter
      real*8 positions(3, numAtoms), velocities(3, numAtoms), simBox
! variables
      integer j
      real*8 tmpPos(3)
! open trajectory file
      if (iter == 1) then
         open(unit=20, file="trj-pos.xyz", status="unknown")
         open(unit=21, file="trj-vel.xyz", status="unknown")
      end if
          
      if (mod(iter, DAMP_TRAJECTORY) == 0) then
         write(20,FMT="(i6)") numAtoms
         write(20,FMT="()") 
         do j = 1, numAtoms
            tmpPos(:) = positions(:, j)
#ifdef DUMPPBC
            tmpPos(:) = tmpPos(:) - simBox * nint(tmpPos(:)/simBox)
#endif
            write(20,FMT="('Ar', 3(1x, e15.6))")  tmpPos(:)
         end do
         write(21,FMT="(i6)") numAtoms
         write(21,FMT="()") 
         do j = 1, numAtoms
            write(21,FMT="('Ar', 3(1x, e15.6))")  velocities(:, j)
         end do
      end if
      if (iter == MAX_ITER) then
         close(20)
         close(21)
      end if
      return
      end
      
      subroutine PrintLogo()
      write(*,*)
      write(*,*) "If it happen to appear this on your monitor" 
      write(*,*) "you are Running..."
      write(*,*) "...MoDERN: Molecular Dynamics Example: Really Naive"
      write(*,*)
      return
      end

      subroutine PrintProgress(iter)
      implicit none
! arguments
      integer iter
! variables
      integer stepIter, nEntry, i
      parameter (stepIter = MAX_ITER/20)
      logical firstCall /.TRUE./
      save firstCall
  
      if (firstCall) then
         firstCall = .FALSE.
         nEntry = MAX_ITER / stepIter -4
         write(*,FMT='("0")',ADVANCE='NO')
         do i = 1, nEntry/2
            write(*,FMT='("-")',ADVANCE='NO')
         end do
         write(*,FMT='("50")',ADVANCE='NO')
         do i = nEntry/2+2, nEntry
            write(*,FMT='("-")',ADVANCE='NO')
         end do
         write(*,FMT='("100")',ADVANCE='NO')
         write(*,*)
      end if
      if(mod(iter,stepIter) == 0) write(0,FMT='("#")',ADVANCE='NO')
      if(iter == MAX_ITER) then 
        write(*,FMT='("#")',ADVANCE='NO')
        write(*,*)
        write(*,*)
        write(*,*) "That's all folks!!!"
        write(*,*)
      endif

      return
      end


      subroutine MyTiming(iter)
      implicit none
! arguments
      integer iter
! variables
      logical firstCall /.TRUE./
      real*8  myCPUTime(2), myWallClock(2)
      save firstCall, myCPUTime, myWallClock
      real*8 DCLOCK
      external DCLOCK
! initial CPU time
      if (firstCall) then
         call CPU_TIME(myCPUTime(1))
!         myWallClock(1) = DCLOCK()
         firstCall = .FALSE.
      else 
         call CPU_TIME(myCPUTime(2))
!         myWallClock(2) = DCLOCK()
         if (iter >= MAX_ITER) then
            write(*,*) "this run took"
            write(*,FMT='("CPU [s]",f10.2)')&
                       myCPUTime(2) - myCPUTime(1)
!            write(*,FMT='("Walltime [s]",f10.2)') &
!                       dble(myWallClock(2) - myWallClock(1)) 
!            write(*,FMT='("Efficiency [%]",f10.2)') &
!                       (myCPUTime(2) - myCPUTime(1)) / &
!                       dble(myWallClock(2) - myWallClock(1)) * &
!                       100.d0
         end if
      end if

      return
      end

       subroutine gaussianNoise(nParticles, mean, variance, noise)
! generates a gaussian distributed random number
! this function works OK only with even number of particle
! the extension to the odd case is very simple

       implicit none
! arguments
       integer nParticles
       double precision noise(3,nParticles)
       double precision mean, variance

! variables
       integer i, j
       double precision u(3,nParticles/2,2)
       double precision, parameter:: pi2 = 4*atan(1.d0)


       u(:,:,1) = 0
       do i = 1, nParticles/2
          do j = 1, 3
             do while (u(j,i,1) == 0)                            ! if U1==0, then the log() below will blow up
                call random_number(u(j,i,1))
             end do
          end do
       end do
       call random_number(u(:,:,2))

       noise(:,1:nParticles/2) = sqrt(-2 * log(u(:,:,1))) * cos(pi2 * u(:,:,2))
       noise(:,nParticles/2+1:nParticles) = sqrt(-2 * log(u(:,:,1))) * sin(pi2 * u(:,:,2))
       noise = mean + sqrt(variance) * noise

       return
       end




       subroutine whiteNoise(nParticles, mean, variance, noise)
! generates a homogeneous distributed random number
! this function works OK only with even number of particle
! the extension to the odd case is very simple

       implicit none
! arguments
       integer nParticles
       double precision noise(3,nParticles)
       double precision mean, variance

       call random_number(noise(:,:))

       noise = mean + sqrt(variance) * noise

       return
       end


      subroutine readFrame (numAtoms, simBox, positions, lastIter)
      implicit none
! arguments
      integer numAtoms
      real*8 positions(3, numAtoms)
      logical lastIter
! variables
      integer j, readAtoms
      character*2 dummy
      real*8 tmpPos(3), simBox

! open trajectory file
         open(unit=20, file="trj-pos.xyz", status="OLD")

!read frame          
         read(20,FMT=*, END=100, ERR=101) readAtoms
         if (readAtoms .ne. numAtoms) stop("wrong number of atoms")
         read(20,FMT=*,ERR=101) 
         do j = 1, numAtoms
            read(20,FMT=*,ERR=101) dummy, tmpPos(:)
#ifdef DUMPPBC
            tmpPos(:) = tmpPos(:) - simBox * nint(tmpPos(:)/simBox)
#endif
            positions(:,j) = tmpPos(:)
         end do

      return

 100  lastIter = .true.
      return

 101  stop("ERROR IN READING DATA")
      close(20)

      end
